//***************************************************************************
// Ces applets viennent en complment du livre :
//
//  		VIE ARTIFICIELLE
//      O la biologie rencontre l'informatique
//		Illustr en Java
//            Jean-Philippe Rennard
//           	  Vuibert 2002
//===========================================================================
//  Automates cellulaires 2D  (Vuibert)
//    v:0.0 30/04/2001
//
//  This program shows the capability of a Conway cellular automata to manage
//  boolean functions.
//
//    Jean-Philippe Rennard 2001-2002
//
//    This program is free software; you can redistribute it and/or
//    modify it under the terms of the GNU General Public License
//    as published by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//***************************************************************************
package ac;

import java.awt.*;
import vabase.*;

/** Classe automate cellulaire */
public class Cac extends Cprocess {
  static final int MAXNBVOISINS=8;
  static final int MAXNBETATS=8;
  static final int TAILLE_DEFAUT=100;
  static final byte VALREPOS=0;
  static final byte VALNAISSANT=1;

  // Figures
  static final int RPENTO=0;
  static final int PLANEUR=1;
  static final int CANONP30=2;
  static final int VAISSEAU=3;
  static final int LOCOMOTIVE=4;
  static final int CODERMAN=5;
  static final byte FIGURES[][][]=
  {
    { // R-pento
      {3,3},
      {0,0,1},
      {1,1,1},
      {0,1,0}
    },
    { // Planeur
      {3,3},
      {0,1,0},
      {0,0,1},
      {1,1,1}
    },
    { // Canon p30
      {36,9},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0},
      {0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,1},
      {0,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,1,1},
      {1,1,0,0,1,1,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0},
      {1,1,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
    },
    { // Vaisseau
      {6,4},
      {0,0,0,0,1,0},
      {0,0,0,0,0,1},
      {1,0,0,0,0,1},
      {0,1,1,1,1,1}
    },
    { // Locomotive
      {5,18},
      {1,0,0,1,0},
      {0,0,0,0,1},
      {1,0,0,0,1},
      {0,1,1,1,1},
      {0,0,0,0,0},
      {0,0,0,0,0},
      {0,0,0,0,0},
      {1,0,0,0,0},
      {0,1,1,0,0},
      {0,0,1,0,0},
      {0,0,1,0,0},
      {0,1,0,0,0},
      {0,0,0,0,0},
      {0,0,0,0,0},
      {1,0,0,1,0},
      {0,0,0,0,1},
      {1,0,0,0,1},
      {0,1,1,1,1}
    },
    { // Puffeur Coderman
      {28,6},
      {0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1},
      {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0}
    },
    { // Planeur 1133
      {5,3},
      {1,1,1,0,0},
      {0,0,0,1,1},
      {1,1,1,0,0}
    }
  };

  /** Pointeur sur l'interface utilisateur */
  private acUI appUI;
  /** Grille de travail 1 */
  private Cgrille grille1;
  /** Grille de travail 2 */
  private Cgrille grille2;
  /** Grille courante (rfrence) */
  Cgrille grille;
  /** Table lookup */
  private byte lookup[][];
  /** Nombre d'tats */
  byte nbEtats;
  /** Vue */
  vueAc acVue;

  /** Constructeur : cre les grilles, init. la vue, etc... */
  public Cac(acUI ui, String regles, int nbCol, int nbLig) {
    super(ui);
    appUI=ui;
    acVue=new vueAc(this, Color.black, Color.white);
    vue=(Cvue)acVue;
    setGrille(nbCol, nbLig);
    setRegles(regles);
  }
  /** Vrification et Fixation des rgles selon chane */
  public boolean setRegles(String r) {
    String paramSurvi, paramNait;
    // vrifie si prsence des sparateurs
    int n=0;
    for(int i=0;i<r.length();i++)
      if(r.charAt(i)=='/') n++;
    if(n!=2) return false;
    // recherche nombre d'tats
    byte nbe=(byte)(r.charAt(r.length()-1)-'0');
    if(nbe<2 || nbe>8) return false;
    if(r.charAt(r.length()-2)!='/') return false;
    nbEtats=nbe;
    // recherche valeurs de survie
    paramSurvi="";
    int i=0;
    char c=r.charAt(i);
    for(i=0; c!='/'; i++) {
      if(c<'0' || c>'9')
        return false;
      paramSurvi += c;
      c=r.charAt(i);
    }
    // recherche valeurs naissance
    paramNait="";
    if(i==0) i++;   // si pas de valeurs de survie (Brian's brain)
    c=r.charAt(i);
    for(int j=i; c!='/'; j++) {
      if(c<'0' || c>'9')
        return false;
      paramNait += c;
      c=r.charAt(j);
    }
    creeLookup(paramSurvi, paramNait);
    return true;
  }
  /** Modification taille */
  public void setTaille(int nbCol, int nbLig) {
    setGrille(nbCol, nbLig);
    initVue();
  }
  /** Initialisation/MAJ de la vue */
  public void initVue() {
    acVue.setNewTaille();
  }
  /** Vide l'automate */
  public void effaceGrille() {
    grille.videGrille();
    generation=0;
    grille.affGrille();
    acVue.repaint();
  }
  /** Construit une grille alatoire. Reoit densit */
  public void grilleAleat(int dens) {
    grille.setGrilleAleat(dens);
    generation=0;
    grille.affGrille();
    acVue.repaint();
  }
  /** Ajoute une figure */
  public void ajoutFigure(int fig) {
    grille.setFigure(fig);
    generation=0;
    grille.affGrille();
    acVue.repaint();
  }
  /** Cration des grilles */
  private void setGrille(int nbCol, int nbLig) {
    grille1 =new Cgrille(this, nbCol, nbLig);
    grille2 =new Cgrille(this, nbCol, nbLig);
    grille=grille1;
    generation=0;
  }
  /** Construit table lookup (rgles) */
  private void creeLookup(String paramSurvi, String paramNait) {
    lookup=null;
    lookup=new byte[nbEtats][MAXNBVOISINS+1];
    // Cas cellule courante morte
    for(int j=0;j<=MAXNBVOISINS;j++)
        lookup[VALREPOS][j]=VALREPOS;
    for (int i=0;i<paramNait.length();i++)
        lookup[VALREPOS][paramNait.charAt(i)-'0']=VALNAISSANT;
    // Cas cellule courante firing
    for(int j=0;j<=MAXNBVOISINS;j++)
        lookup[VALNAISSANT][j]=(byte)((VALNAISSANT+1)%nbEtats);
    for (int i=0;i<paramSurvi.length();i++)
        lookup[VALNAISSANT][paramSurvi.charAt(i)-'0']=VALNAISSANT;
    // Autres cas
    for (int i=VALNAISSANT+1;i<nbEtats;i++)
      for (int j=0;j<=MAXNBVOISINS;j++)
        lookup[i][j]=(byte)((i+1) % nbEtats);
  }
  /** Gnration suivante */
  public void geneSuiv() {
  // switch grille courante
    Cgrille grBuff;
    grBuff=(grille==grille1 ? grille2 : grille1);
    for(int i=0;i<grille.getNbCellHoriz();i++) {
      for(int j=0;j<grille.getNbCellVertic();j++) {
        grBuff.setCellule(i,j,lookup[grille.getCellule(i,j)]
                 [grille.getNbVoisins(i,j)]);
        if (grBuff.getCellule(i,j)!=grille.getCellule(i,j))
           acVue.affCell(i, j, grBuff.getCellule(i,j));
      }
    }
    generation++;
    grille=grBuff;
    acVue.repaint();
    appUI.affGeneration();
  }
}
