//***************************************************************************
// Ces applets viennent en complment du livre :
//
//  		VIE ARTIFICIELLE
//      O la biologie rencontre l'informatique
//		Illustr en Java
//            Jean-Philippe Rennard
//           	  Vuibert 2002
//===========================================================================
//  Automates cellulaires 2D  (Vuibert)
//    v:0.0 30/04/2001
//
//  This program shows the capability of a Conway cellular automata to manage
//  boolean functions.
//
//    Jean-Philippe Rennard 2001-2002
//
//    This program is free software; you can redistribute it and/or
//    modify it under the terms of the GNU General Public License
//    as published by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//***************************************************************************
package ac;

/** Cette classe gre une grille  2 dimensions */
public class Cgrille {
  static final int NORD=1;
  static final int NORDEST=2;
  static final int EST=3;
  static final int SUDEST=4;
  static final int SUD=5;
  static final int SUDOUEST=6;
  static final int OUEST=7;
  static final int NORDOUEST=8;

  /** Pointeur sur l'automate cellulaire */
  private Cac acPere;
  /** Table des cellules */
  private byte cellule[][];
  /** Paramtres de dimension */
  private int tailleX, tailleY;

  /** Constructeur de la grille */
  public Cgrille(Cac ap, int x, int y) {
    acPere=ap;
    cellule=new byte[x][y];
    tailleX=x;
    tailleY=y;
    videGrille();
  }
  /** Renvoie nombre de cellules horizontales */
  public int getNbCellHoriz() {
    return(tailleX);
  }
  /** Renvoie nombre de cellules verticales */
  public int getNbCellVertic() {
    return(tailleY);
  }
  /** Passe une cellule  1 tat donn */
  public void setCellule(int x, int y, byte etat) {
    cellule[x][y]=etat;
  }
  /** Retourne l'tat de la cellule */
  public int getCellule(int x, int y) {
    return(cellule[x][y]);
  }
  /** Affiche une cellule */
  public void affCell(int x, int y) {
    acPere.acVue.affCell(x, y, cellule[x][y]);
  }
  /** Affiche l'ensemble des cellules */
  public void affGrille() {
    acPere.acVue.peintFond();
    for(int x=0;x<tailleX;x++)
      for(int y=0;y<tailleY;y++)
            affCell(x,y);
  }
  /** Vide la grille (tout  l'tat repos) */
  public void videGrille() {
    for(int x=0;x<tailleX;x++)
      for(int y=0;y<tailleY;y++)
        setCellule(x, y, Cac.VALREPOS);
  }

  /** Remplissage d'une grille avec des valeurs alatoires. Reoit densit */
  public void setGrilleAleat(int dens) {
    videGrille();
    for(int x=0;x<tailleX;x++)
      for(int y=0;y<tailleY;y++)
        if(Math.random()<(dens/100.0))
          setCellule(x,y,Cac.VALNAISSANT);
        else
          setCellule(x,y,Cac.VALREPOS);
  }
  /** Ajoute une figure */
  public void setFigure(int fig) {
    videGrille();
    int l=Cac.FIGURES[fig][0][0];
    int c=Cac.FIGURES[fig][0][1];
    int posx=tailleX/2-(l/2);
    int posy=tailleY/2-(c/2);
    for(int i=1;i<=c;i++)
      for(int j=0;j<l;j++)
        cellule[posx+j][posy+i]=Cac.FIGURES[fig][i][j];
  }
  /** Retourne nombre de voisins actifs (tat VALNAISSANT) */
  public int getNbVoisins(int x, int y) {
    int n=0;
    if(getVoisin(x,y,NORD)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,NORDEST)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,EST)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,SUDEST)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,SUD)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,SUDOUEST)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,OUEST)==Cac.VALNAISSANT) n++;
    if(getVoisin(x,y,NORDOUEST)==Cac.VALNAISSANT) n++;
    return n;
  }

  /** Retourne l'tat d'une cellule selon position */
  private int getVoisin(int x, int y, int pos) {
    switch(pos) {
      case NORD:
        return(cellule[x][(tailleY+y-1)%tailleY]);
      case NORDEST:
        return(cellule[(x+1)%tailleX][(tailleY+y-1)%tailleY]);
      case EST:
        return(cellule[(x+1)%tailleX][y]);
      case SUDEST:
        return(cellule[(x+1)%tailleX][(tailleY+y+1)%tailleY]);
      case SUD:
        return(cellule[x][(tailleY+y+1)%tailleY]);
      case SUDOUEST:
        return(cellule[(tailleX+x-1)%tailleX][(tailleY+y+1)%tailleY]);
      case OUEST:
        return(cellule[(tailleX+x-1)%tailleX][y]);
      case NORDOUEST:
        return(cellule[(tailleX+x-1)%tailleX][(tailleY+y-1)%tailleY]);
      default:
        return cellule[x][y];
    }
  }
}