//***************************************************************************
// Ces applets viennent en complment du livre :
//
//  		VIE ARTIFICIELLE
//      O la biologie rencontre l'informatique
//		Illustr en Java
//            Jean-Philippe Rennard
//           	  Vuibert 2002
//===========================================================================
//  Automates cellulaires 2D  (Vuibert)
//    v:0.0 30/04/2001
//
//  This program shows the capability of a Conway cellular automata to manage
//  boolean functions.
//
//    Jean-Philippe Rennard 2001-2002
//
//    This program is free software; you can redistribute it and/or
//    modify it under the terms of the GNU General Public License
//    as published by the Free Software Foundation.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//***************************************************************************
package ac;

import java.awt.*;
import vabase.*;

/** Classe de gestion de la vue graphique */
public class vueAc extends Cvue {

  /** Pointeur sur l'automate */
  private Cac acPere;
  /** Taille d'une cellule */
  private float tailleCellHoriz, tailleCellVertic;
  private float offsTailleCaseH, offsTailleCaseV;
  private int largeurCell, hauteurCell;
  /** Nombre de cellules affiches */
  private int nbCellHoriz, nbCellVertic;
  /** Table des couleurs */
  private Color couleur[];

  /** Constructeur par dfaut */
  public vueAc(Cac ac) {
    super(Color.black, Color.white, Cvue.REPRANISOTROPIC);
    acPere=ac;
    setCouleur();
  }
  /** Constructeur avec fond */
  public vueAc(Cac ac, Color c, Color f) {
    super(c, f, Cvue.REPRANISOTROPIC);
    acPere=ac;
    setCouleur();
  }

  /** Fixe nouvelle taille */
  public void setNewTaille() {
    redimGrBuff();
    setTailleCell(acPere.grille.getNbCellHoriz(), acPere.grille.getNbCellVertic());
    peintFond();
    acPere.grille.affGrille();
    repaint();
  }
  /** Fixe la taille des cellules, reoit le nombre horiz/vertic */
  public void setTailleCell(int nbh, int nbv) {
    nbCellHoriz=nbh;
    nbCellVertic=nbv;
    tailleCellHoriz=lLargeur/nbCellHoriz;
    tailleCellVertic=lHauteur/nbCellVertic;
    offsTailleCaseH= (tailleCellHoriz>4.0F ? 1.0F : 0.0F);
    offsTailleCaseV= (tailleCellVertic>4.0F ? 1.0F : 0.0F);
    largeurCell=Math.max(lLongueurToRX(tailleCellHoriz-2*offsTailleCaseH)-1,1);
    hauteurCell=Math.max(lLongueurToRY(tailleCellVertic-2*offsTailleCaseV)-1,1);
  }

  /** Affiche une cellule */
  public void affCell(int x, int y, int et) {
    grBuff.setColor(couleur[et]);
    grBuff.fillRect(lToRX(x*tailleCellHoriz+offsTailleCaseH)+1,
                      lToRY(y*tailleCellVertic+offsTailleCaseV)+1,
                      largeurCell, hauteurCell);
  }

  /** Dessine le fond */
  public void peintFond() {
    effaceFond();
    Color colcrt=grBuff.getColor();
    grBuff.setColor(colcrt);
    grBuff.drawRect(lToRX(0),lToRY(0),aLargeur,aHauteur);
    grBuff.setColor(colcrt);
    if(lLongueurToRX(tailleCellHoriz)>5 && lLongueurToRY(tailleCellVertic)>5) {
      for(int i=0;i<=nbCellHoriz;i++)
        grBuff.drawLine(lToRX(i*tailleCellHoriz),lToRY(0),
                          lToRX(i*tailleCellHoriz),
                          lToRY(nbCellVertic*tailleCellVertic));
      for(int i=0;i<=nbCellVertic;i++)
        grBuff.drawLine(lToRX(0), lToRY(i*tailleCellVertic),
                          lToRX(nbCellHoriz*tailleCellHoriz),
                          lToRY(i*tailleCellVertic));
    }
  }

  /** Init. table couleurs */
  private void setCouleur() {
    couleur=new Color[Cac.MAXNBETATS];
    couleur[0]=Color.white;
    couleur[1]=Color.black;
    couleur[2]=Color.red;
    couleur[3]=Color.magenta;
    couleur[4]=Color.pink;
    couleur[5]=Color.orange;
    couleur[6]=Color.yellow;
    couleur[7]=Color.green;
  }
  /** Modification d'une cellule avec la souris. Passe  l'tat suivant */
  public boolean mouseDown(Event e,int x,int y) {
    if(acPere.isRunning)
      return(false);

    int posx=(int)(rToLX(x)/tailleCellHoriz);
    int posy=(int)(rToLY(y)/tailleCellVertic);
    if(posx<0 || posx>acPere.grille.getNbCellHoriz()-1 ||
          posy<0 || posy>acPere.grille.getNbCellVertic()-1)
            return(false);
    acPere.grille.setCellule(posx, posy,
              (byte)((acPere.grille.getCellule(posx, posy)+1)%acPere.nbEtats));
    acPere.grille.affCell(posx, posy);
    repaint();
    return(true);
  }
}
